/**
 * @file 	interrupts.c
 * @version 	0.01
 * @date 	Feb 25, 2013
 * @author 	Grzegorz Kostka, kostka.grzegorz@gmail.com
 * @brief	...
 **********************************************************/

#include "interrupts.h"
#include "sharc_regs.h"

#include <platform_include.h>
#include <services/int/adi_int.h>

/**@brief	Mapa wektorow przerwan.*/
static uint32_t vects[VECT_TOP] = {
		ADI_CID_IICDI, ADI_CID_SOVFI, ADI_CID_TMZHI, ADI_CID_SPERRI, ADI_CID_BKPI,
		ADI_CID_IRQ2I, ADI_CID_IRQ1I, ADI_CID_IRQ0I, ADI_CID_P0I,    ADI_CID_P1I,
		ADI_CID_P2I,   ADI_CID_P3I,   ADI_CID_P4I,   ADI_CID_P5I,    ADI_CID_P6I,
		ADI_CID_P7I,   ADI_CID_P8I,   ADI_CID_P9I,   ADI_CID_P10I,   ADI_CID_P11I,
		ADI_CID_P12I,  ADI_CID_P13I,  ADI_CID_P14I,  ADI_CID_P15I,   ADI_CID_P16I,
		ADI_CID_P17I,  ADI_CID_P18I,  ADI_CID_CB7I,  ADI_CID_CB15I,  ADI_CID_TMZLI,
		ADI_CID_FIXI,  ADI_CID_FLTOI, ADI_CID_FLTUI, ADI_CID_FLTII,  ADI_CID_EMULI,
		ADI_CID_SFT0I, ADI_CID_SFT1I, ADI_CID_SFT2I, ADI_CID_SFT3I,
};

static enum irq_vec assign_tab[PIRQ_SRC_TOP] = {
		PVECT_0,  PVECT_1,  PVECT_2,  PVECT_3,  PVECT_4,  PVECT_5,  PVECT_6,
		PVECT_7,  PVECT_8,  PVECT_9,  PVECT_10, PVECT_11, PVECT_12, PVECT_13,
		PVECT_14, PVECT_15, PVECT_16, PVECT_17, PVECT_18,

		VECT_TOP, VECT_TOP, VECT_TOP, VECT_TOP, VECT_TOP, VECT_TOP, VECT_TOP,
		VECT_TOP, VECT_TOP, VECT_TOP, VECT_TOP, VECT_TOP, VECT_TOP,
};

struct irq_internals {
	void *picr;
};

static struct irq_internals irq_int = {
		.picr = (void *)PICR0,
};

uint32_t irq_assign_set(enum pirq_src src, enum irq_vec vect)
{

	uint32_t tid;
	uint32_t bid;
	uint32_t *p = irq_int.picr;

	if ((vect < PVECT_0) || (vect > PVECT_18))
		return 0;

	tid = (vect - PVECT_0) / 6;
	bid = (vect - PVECT_0) % 6;

	sharc_reg_modify(p + tid, 0x1F << (bid * 5), src << (bid * 5));
	assign_tab[src] = vect;
	return 1;
}

enum irq_vec irq_assign_get(enum pirq_src src) { return assign_tab[src]; }

uint32_t irq_install(enum irq_vec vect, interrupt_t irq)
{

	return ADI_INT_SUCCESS ==
			adi_int_InstallHandler(vects[vect], (ADI_INT_HANDLER_PTR)irq, 0,
					1);
}
