/**
 * @file 	interrupts.h
 * @version 	0.01
 * @date 	Feb 25, 2013
 * @author 	Grzegorz Kostka, kostka.grzegorz@gmail.com
 * @brief	...
 **********************************************************/

#ifndef INTERRUPTS_H_
#define INTERRUPTS_H_

#include <stdint.h>

enum pirq_src {
	/*DEFAULT*/
	PIRQ_SRC_DAIHI = 0,
	PIRQ_SRC_SPIHI,
	PIRQ_SRC_GPTMR0I,
	PIRQ_SRC_SP1I,
	PIRQ_SRC_SP3I,
	PIRQ_SRC_SP5I,
	PIRQ_SRC_SP0I,
	PIRQ_SRC_SP2I,
	PIRQ_SRC_SP4I,
	PIRQ_SRC_EPDM0I,
	PIRQ_SRC_GPTMR1I,
	PIRQ_SRC_SP7I,
	PIRQ_SRC_DAILI,
	PIRQ_SRC_EPDM1I,
	PIRQ_SRC_DPII,
	PIRQ_SRC_MTMI,
	PIRQ_SRC_SP6I,
	PIRQ_SRC_DISABLED0,
	PIRQ_SRC_SPILI,
	/*EXTRA*/
	PIRQ_SRC_UART0RxI,
	PIRQ_SRC_DISABLED1,
	PIRQ_SRC_UART0TxI,
	PIRQ_SRC_DISABLED2,
	PIRQ_SRC_TWII,
	PIRQ_SRC_PWMI,
	PIRQ_SRC_LP0I_RTCI,
	PIRQ_SRC_LP1I,
	PIRQ_SRC_ACC0I,
	PIRQ_SRC_ACC1I,
	PIRQ_SRC_MLBI,
	PIRQ_SRC_DISABLED3,
	PIRQ_SRC_SOFTWARE,

	PIRQ_SRC_TOP,
};

enum irq_vec {

	VECT_IICDI = 0,
	VECT_SOVFI,
	VECT_TMZHI,
	VECT_SPERRI,
	VECT_BKPI,
	VECT_IRQ2I,
	VECT_IRQ1I,
	VECT_IRQ0I,

	/*PROGRAMABLE*/
	PVECT_0,
	PVECT_1,
	PVECT_2,
	PVECT_3,
	PVECT_4,
	PVECT_5,
	PVECT_6,
	PVECT_7,
	PVECT_8,
	PVECT_9,
	PVECT_10,
	PVECT_11,
	PVECT_12,
	PVECT_13,
	PVECT_14,
	PVECT_15,
	PVECT_16,
	PVECT_17,
	PVECT_18,

	VECT_CB7I,
	VECT_CB15I,
	VECT_TMZLI,
	VECT_FIXI,
	VECT_FLTOI,
	VECT_FLTUI,
	VECT_FLTII,
	VECT_EMULI,
	VECT_SFT0I,
	VECT_SFT1I,
	VECT_SFT2I,
	VECT_SFT3I,

	VECT_TOP,
};

#define IRQ_REENTRANT_HANDLER(__name) void __name(void)

typedef void (*interrupt_t)(void);

uint32_t irq_assign_set(enum pirq_src src, enum irq_vec vect);
enum irq_vec irq_assign_get(enum pirq_src src);
uint32_t irq_install(enum irq_vec vect, interrupt_t irq);

#endif /* INTERRUPTS_H_ */

